/*
 *
 * Stratum (Updated)
 *
 */

const redis = require('redis-mock');
const mockStratum = require('./stratum.mock.js');
const PoolLogger = require('../main/logger');
const PoolShares = require('../main/shares');
const PoolStratum = require('../main/stratum');

const poolConfig = {
    "address": "tb1qcc0lzt4fftzmpxuye6q8vnfngu03yuwpasu0dw",
    "coin": {
        "name": "Bitcoin",
        "algorithm": "sha256d",
        "peerMagic": "f9beb4d9",
        "peerMagicTestnet": "0b110907",
        "mainnet": {
            "bech32": "bc",
            "bip32": {
                "public": "0488B21E",
                "private": "0488ADE4"
            },
            "pubKeyHash": "00",
            "scriptHash": "05",
            "wif": "80",
            "coin": "btc"
        },
        "testnet": {
            "bech32": "tb",
            "bip32": {
                "public": "043587CF",
                "private": "04358394"
            },
            "pubKeyHash": "6F",
            "scriptHash": "C4",
            "wif": "EF",
            "coin": "btc"
        }
    },
    "daemons": [{
        "host": "127.0.0.1",
        "port": 8332,
        "user": "",
        "password": ""
    }],
    "logColors": true,
    "logLevel": "debug",
    "ports": {
        "3001": {
            "soloMining": true,
        }
    },
    "p2p": {
        "enabled": true,
        "host": "127.0.0.1",
        "port": 8333,
        "disableTransactions": true
    },
    "recipients": [
        {
            "address": "tb1qcc0lzt4fftzmpxuye6q8vnfngu03yuwpasu0dw",
            "percentage": 0.05
        }
    ],
}

const portalConfig = {
    "redis": {
        "host": "127.0.0.1",
        "port": "6379",
    }
}

const client = redis.createClient({
    "port": portalConfig.redis.port,
    "host": portalConfig.redis.host,
});
client._redisMock._maxListeners = 0;

const logger = new PoolLogger(poolConfig);
const poolShares = new PoolShares(logger, client, poolConfig, portalConfig)

////////////////////////////////////////////////////////////////////////////////

describe('Test stratum functionality', () => {

    test('Test start', () => {
        mockStratum.mockDaemon();
        const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
        poolStratum.start();
    })

    // test('Test block viability checker [1]', () => {
    //     const consoleSpy = jest.spyOn(console, 'log').mockImplementation(() => {});
    //     const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
    //     poolStratum.checkBlock({ hash: "example" }, false);
    //     expect(consoleSpy).toHaveBeenCalledWith(expect.stringMatching('We thought a block was found but it was rejected by the daemon'));
    // });
    //
    // test('Test block viability checker [2]', () => {
    //     const consoleSpy = jest.spyOn(console, 'log').mockImplementation(() => {});
    //     const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
    //     poolStratum.checkBlock({ hash: "example" }, true);
    //     expect(consoleSpy).toHaveBeenCalledWith(expect.stringMatching('Block found'));
    // });
    //
    // test('Test share viability checker [1]', () => {
    //     const consoleSpy = jest.spyOn(console, 'log').mockImplementation(() => {});
    //     const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
    //     poolStratum.checkShare({}, false);
    //     expect(consoleSpy).toHaveBeenCalledWith(expect.stringMatching('Share rejected by the daemon'));
    // });
    //
    // test('Test share viability checker [2]', () => {
    //     const consoleSpy = jest.spyOn(console, 'log').mockImplementation(() => {});
    //     const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
    //     poolStratum.checkShare({}, true);
    //     expect(consoleSpy).toHaveBeenCalledWith(expect.stringMatching('Share accepted at difficulty'));
    // });
    //
    // test('Test worker authorization method [1]', () => {
    //     const poolStratum = new PoolStratum(logger, poolConfig, poolShares);
    // });
});
